@@ -1,4 +1,4 @@
-/*      $Id: compositor.h 27454 2008-08-15 21:45:57Z olivier $
+/*      $Id: compositor.h 27453 2008-08-15 20:56:06Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         xcompmgr - (c) 2003 Keith Packard
-        xfwm4    - (c) 2005-2006 Olivier Fourdan
+        xfwm4    - (c) 2005-2008 Olivier Fourdan
 
  */
 
@@ -23,7 +25,7 @@
 #define INC_COMPOSITOR_H
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
