@@ -1,4 +1,4 @@
-/*      $Id: frame.c 25371 2007-04-02 19:32:01Z olivier $
+/*      $Id: frame.c 27001 2008-05-31 23:41:10Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <glib.h>
@@ -37,9 +39,12 @@
 #include "frame.h"
 #include "compositor.h"
 
-#ifndef ShapeInput
-#define ShapeInput 2;
-#endif
+typedef struct
+{
+    xfwmPixmap pm_title;
+    xfwmPixmap pm_sides[SIDE_COUNT];
+} FramePixmap;
+
 
 int
 frameDecorationLeft (ScreenInfo *screen_info)
@@ -249,7 +254,31 @@ frameButtonOffset (Client *c)
 }
 
 static void
-frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap * pm)
+frameFillTitlePixmap (Client * c, int state, int part, int x, int w, int h, xfwmPixmap * title_pm, xfwmPixmap * top_pm)
+{
+    ScreenInfo *screen_info;
+
+    TRACE ("entering frameFillTitlePixmap");
+
+    g_return_if_fail (c);
+    g_return_if_fail (title_pm);
+    g_return_if_fail (top_pm);
+
+    screen_info = c->screen_info;
+
+    if (!xfwmPixmapNone(&screen_info->top[part][state]))
+    {
+        xfwmPixmapFill (&screen_info->top[part][state], top_pm, x, 0, w, h);
+    }
+    else
+    {
+        xfwmPixmapFill (&screen_info->title[part][state], top_pm, x, 0, w, h);
+    }
+    xfwmPixmapFill (&screen_info->title[part][state], title_pm, x, 0, w, frameTop (c));
+}
+
+static void
+frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap * title_pm, xfwmPixmap * top_pm)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -260,11 +289,13 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     PangoRectangle logical_rect;
     int width, x, hoffset, w1, w2, w3, w4, w5, temp;
     int voffset, title_x, title_y;
+    int title_height, top_height;
 
     TRACE ("entering frameCreateTitlePixmap");
 
     g_return_if_fail (c);
-    g_return_if_fail (pm);
+    g_return_if_fail (title_pm);
+    g_return_if_fail (top_pm);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
@@ -279,13 +310,12 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     width = frameWidth (c) - frameTopLeftWidth (c, state) - frameTopRightWidth (c, state);
     if (width < 1)
     {
-        pm->pixmap = None;
-        pm->mask = None;
-        pm->width = 0;
-        pm->height = 0;
         return;
     }
 
+    layout = gtk_widget_create_pango_layout (myScreenGetGtkWidget (screen_info), c->name);
+    pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+
     if (left < frameTopLeftWidth (c, state))
     {
         left = frameTopLeftWidth (c, state);
@@ -304,15 +334,47 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
 
     x = 0;
     hoffset = 0;
-    voffset = 0;
+
+    if (state == ACTIVE)
+    {
+        voffset = screen_info->params->title_vertical_offset_active;
+    }
+    else
+    {
+        voffset = screen_info->params->title_vertical_offset_inactive;
+    }
+
+    title_height = screen_info->font_height;
+    if (!title_height)
+    {
+        /* If for some reason the font height is not known,
+         * use the actual pango layout height.
+         */
+        title_height = logical_rect.height;
+    }
+    title_y = voffset + (frameTop (c) - title_height) / 2;
+    if (title_y + title_height > frameTop (c))
+    {
+        title_y = MAX (0, frameTop (c) - title_height);
+    }
+
+    if (!xfwmPixmapNone(&screen_info->top[3][ACTIVE]))
+    {
+        top_height = screen_info->top[3][ACTIVE].height;
+    }
+    else
+    {
+        top_height = frameTop (c) / 10 + 1;
+        if (top_height > title_y - 1)
+        {
+             top_height = MAX (title_y - 1, 0);
+        }
+    }
 
     w1 = 0;
     w2 = screen_info->title[TITLE_2][state].width;
     w4 = screen_info->title[TITLE_4][state].width;
 
-    layout = gtk_widget_create_pango_layout (myScreenGetGtkWidget (screen_info), c->name);
-    pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-
     if (screen_info->params->full_width_title)
     {
         w1 = left;
@@ -341,7 +403,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     }
     else
     {
-        w3 = logical_rect.width;
+        w3 = logical_rect.width + screen_info->params->title_shadow[state];
         w5 = width;
         if (w3 > width - w2 - w4)
         {
@@ -369,37 +431,25 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
         }
     }
 
-    xfwmPixmapCreate (screen_info, pm, width, frameTop (c));
-    gpixmap = gdk_pixmap_foreign_new (pm->pixmap);
+    xfwmPixmapCreate (screen_info, top_pm, width, top_height);
+    xfwmPixmapCreate (screen_info, title_pm, width, frameTop (c));
+    gpixmap = gdk_pixmap_foreign_new (title_pm->pixmap);
     gdk_drawable_set_colormap (gpixmap, gdk_screen_get_rgb_colormap (screen_info->gscr));
     gc = gdk_gc_new (gpixmap);
 
     if (w1 > 0)
     {
-        xfwmPixmapFill (&screen_info->title[TITLE_1][state], pm, 0, 0, w1, frameTop (c));
+        frameFillTitlePixmap (c, state, TITLE_1, x, w1, top_height, title_pm, top_pm);
         x = x + w1;
     }
 
-    xfwmPixmapFill (&screen_info->title[TITLE_2][state], pm, x, 0, w2, frameTop (c));
+    frameFillTitlePixmap (c, state, TITLE_2, x, w2, top_height, title_pm, top_pm);
     x = x + w2;
 
     if (w3 > 0)
     {
-        if (state == ACTIVE)
-        {
-            voffset = screen_info->params->title_vertical_offset_active;
-        }
-        else
-        {
-            voffset = screen_info->params->title_vertical_offset_inactive;
-        }
-        xfwmPixmapFill (&screen_info->title[TITLE_3][state], pm, x, 0, w3, frameTop (c));
+        frameFillTitlePixmap (c, state, TITLE_3, x, w3, top_height, title_pm, top_pm);
         title_x = hoffset + x;
-        title_y = voffset + (frameTop (c) - logical_rect.height) / 2;
-        if (title_y + logical_rect.height > frameTop (c))
-        {
-            title_y = MAX (0, frameTop (c) - logical_rect.height);
-        }
         if (screen_info->params->title_shadow[state])
         {
             gdk_gc_get_values (screen_info->title_shadow_colors[state].gc, &values);
@@ -426,12 +476,12 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     {
         x = right - w4;
     }
-    xfwmPixmapFill (&screen_info->title[TITLE_4][state], pm, x, 0, w4, frameTop (c));
+    frameFillTitlePixmap (c, state, TITLE_4, x, w4, top_height, title_pm, top_pm);
     x = x + w4;
 
     if (w5 > 0)
     {
-        xfwmPixmapFill (&screen_info->title[TITLE_5][state], pm, x, 0, w5, frameTop (c));
+        frameFillTitlePixmap (c, state, TITLE_5, x, w5, top_height, title_pm, top_pm);
     }
     g_object_unref (G_OBJECT (gc));
     g_object_unref (G_OBJECT (gpixmap));
@@ -539,51 +589,6 @@ getLetterFromButton (int i, Client * c)
     return chr;
 }
 
-static xfwmPixmap *
-frameGetPixmap (Client * c, int button, int state)
-{
-    ScreenInfo *screen_info;
-
-    screen_info = c->screen_info;
-    switch (button)
-    {
-        case MENU_BUTTON:
-            if (screen_info->params->show_app_icon)
-            {
-                return &c->appmenu[state];
-            }
-            break;
-        case SHADE_BUTTON:
-            if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED)
-                && screen_info->buttons[SHADE_BUTTON][state + 3].pixmap)
-            {
-                return &screen_info->buttons[SHADE_BUTTON][state + 3];
-            }
-            return &screen_info->buttons[SHADE_BUTTON][state];
-            break;
-        case STICK_BUTTON:
-            if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY)
-                && screen_info->buttons[STICK_BUTTON][state + 3].pixmap)
-            {
-                return &screen_info->buttons[STICK_BUTTON][state + 3];
-            }
-            return &screen_info->buttons[STICK_BUTTON][state];
-            break;
-        case MAXIMIZE_BUTTON:
-            if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED)
-                && screen_info->buttons[MAXIMIZE_BUTTON][state + 3].pixmap)
-            {
-                return &screen_info->buttons[MAXIMIZE_BUTTON][state + 3];
-            }
-            return &screen_info->buttons[MAXIMIZE_BUTTON][state];
-            break;
-        default:
-            break;
-    }
-    return &screen_info->buttons[button][state];
-}
-
-
 static void
 frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTTON_COUNT])
 {
@@ -653,6 +658,12 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
                                ShapeBounding, 0, 0, frame_pix->pm_sides[SIDE_BOTTOM].mask, ShapeSet);
         }
 
+        if (xfwmWindowVisible (&c->sides[SIDE_TOP]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_TOP]),
+                               ShapeBounding, 0, 0, frame_pix->pm_sides[SIDE_TOP].mask, ShapeSet);
+        }
+
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
         {
             XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
@@ -677,11 +688,11 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
                                ShapeBounding, 0, 0, screen_info->corners[CORNER_TOP_RIGHT][state].mask, ShapeSet);
         }
 
-        for (i = 0; i < BUTTON_LAST; i++)
+        for (i = 0; i < BUTTON_COUNT; i++)
         {
             if (xfwmWindowVisible (&c->buttons[i]))
             {
-                my_pixmap = frameGetPixmap (c, i, c->button_pressed[i] ? PRESSED : state);
+                my_pixmap = clientGetButtonPixmap (c, i, clientGetButtonState (c, i, state));
                 XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->buttons[i]),
                                    ShapeBounding, 0, 0, my_pixmap->mask, ShapeSet);
             }
@@ -772,6 +783,14 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
                                 MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]), ShapeBounding, ShapeUnion);
         }
 
+        if (xfwmWindowVisible (&c->sides[SIDE_TOP]))
+        {
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding,
+                                screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
+                                frameTop (c) - frameBottom (c),
+                                MYWINDOW_XWINDOW (c->sides[SIDE_TOP]), ShapeBounding, ShapeUnion);
+        }
+
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
         {
             XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, 0,
@@ -794,7 +813,7 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
                                 0, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding, ShapeUnion);
         }
 
-        for (i = 0; i < BUTTON_LAST; i++)
+        for (i = 0; i < BUTTON_COUNT; i++)
         {
             if (xfwmWindowVisible (&c->buttons[i]))
             {
@@ -821,8 +840,8 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
     XDestroyWindow (display_info->dpy, shape_win);
 }
 
-void
-frameDraw (Client * c, gboolean clear_all)
+static void
+frameDrawWin (Client * c)
 {
     ScreenInfo *screen_info;
     FramePixmap frame_pix;
@@ -839,6 +858,8 @@ frameDraw (Client * c, gboolean clear_all)
 
     g_return_if_fail (c != NULL);
 
+    frameClearQueueDraw (c);
+
     screen_info = c->screen_info;
     requires_clearing = FALSE;
     width_changed = FALSE;
@@ -882,18 +903,12 @@ frameDraw (Client * c, gboolean clear_all)
     FLAG_UNSET (c->xfwm_flags,  XFWM_FLAG_FIRST_MAP);
 
     /* Cache mgmt */
-    if (clear_all)
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_NEEDS_REDRAW))
     {
+        FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_NEEDS_REDRAW);
         width_changed = TRUE;
         height_changed = TRUE;
         requires_clearing = TRUE;
-        frameClearQueueDraw (c);
-        if (c->frame_timeout_id)
-        {
-            g_source_remove (c->frame_timeout_id);
-            c->frame_timeout_id = 0;
-        }
-
     }
     else
     {
@@ -913,7 +928,7 @@ frameDraw (Client * c, gboolean clear_all)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         /* First, hide the buttons that we don't have... */
-        for (i = 0; i < BUTTON_LAST; i++)
+        for (i = 0; i < BUTTON_COUNT; i++)
         {
             char b = getLetterFromButton (i, c);
             if ((!b) || !strchr (screen_info->params->button_layout, b))
@@ -940,8 +955,8 @@ frameDraw (Client * c, gboolean clear_all)
             {
                 if (x + screen_info->buttons[button][state].width + screen_info->params->button_spacing < right)
                 {
-                    my_pixmap = frameGetPixmap (c, button, c->button_pressed[button] ? PRESSED : state);
-                    if (my_pixmap->pixmap)
+                    my_pixmap = clientGetButtonPixmap (c, button, clientGetButtonState (c, button, state));
+                    if (!xfwmPixmapNone(my_pixmap))
                     {
                         xfwmWindowSetBG (&c->buttons[button], my_pixmap);
                     }
@@ -975,8 +990,8 @@ frameDraw (Client * c, gboolean clear_all)
             {
                 if (x - screen_info->buttons[button][state].width - screen_info->params->button_spacing > left)
                 {
-                    my_pixmap = frameGetPixmap (c, button, c->button_pressed[button] ? PRESSED : state);
-                    if (my_pixmap->pixmap)
+                    my_pixmap = clientGetButtonPixmap (c, button, clientGetButtonState (c, button, state));
+                    if (!xfwmPixmapNone(my_pixmap))
                     {
                         xfwmWindowSetBG (&c->buttons[button], my_pixmap);
                     }
@@ -1007,12 +1022,13 @@ frameDraw (Client * c, gboolean clear_all)
             screen_info->corners[CORNER_BOTTOM_RIGHT][state].height;
 
         xfwmPixmapInit (screen_info, &frame_pix.pm_title);
+        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_TOP]);
+        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_BOTTOM]);
         xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_LEFT]);
         xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_RIGHT]);
-        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_BOTTOM]);
 
         /* The title is always visible */
-        frameCreateTitlePixmap (c, state, left, right, &frame_pix.pm_title);
+        frameCreateTitlePixmap (c, state, left, right, &frame_pix.pm_title, &frame_pix.pm_sides[SIDE_TOP]);
         xfwmWindowSetBG (&c->title, &frame_pix.pm_title);
         xfwmWindowShow (&c->title,
             frameTopLeftWidth (c, state), 0, top_width,
@@ -1037,6 +1053,7 @@ frameDraw (Client * c, gboolean clear_all)
             xfwmWindowHide (&c->sides[SIDE_LEFT]);
             xfwmWindowHide (&c->sides[SIDE_RIGHT]);
             xfwmWindowHide (&c->sides[SIDE_BOTTOM]);
+            xfwmWindowHide (&c->sides[SIDE_TOP]);
             xfwmWindowHide (&c->corners[CORNER_TOP_LEFT]);
             xfwmWindowHide (&c->corners[CORNER_TOP_RIGHT]);
             xfwmWindowHide (&c->corners[CORNER_BOTTOM_LEFT]);
@@ -1085,6 +1102,19 @@ frameDraw (Client * c, gboolean clear_all)
                 frameHeight (c) - frameBottom (c), bottom_width, frameBottom (c),
                 (requires_clearing | width_changed));
 
+            if (!xfwmPixmapNone(&frame_pix.pm_sides[SIDE_TOP]))
+            {
+                xfwmWindowSetBG (&c->sides[SIDE_TOP], &frame_pix.pm_sides[SIDE_TOP]);
+                xfwmWindowShow (&c->sides[SIDE_TOP],
+                    screen_info->corners[CORNER_TOP_LEFT][state].width,
+                    0, top_width, frame_pix.pm_sides[SIDE_TOP].height,
+                    (requires_clearing | width_changed));
+            }
+            else
+            {
+                xfwmWindowHide (&c->sides[SIDE_TOP]);
+            }
+
             xfwmWindowShow (&c->corners[CORNER_TOP_LEFT], 0, 0,
                 frameTopLeftWidth (c, state),
                 screen_info->corners[CORNER_TOP_LEFT][state].height,
@@ -1113,7 +1143,9 @@ frameDraw (Client * c, gboolean clear_all)
                 requires_clearing);
         }
         frameSetShape (c, state, &frame_pix, button_x);
+
         xfwmPixmapFree (&frame_pix.pm_title);
+        xfwmPixmapFree (&frame_pix.pm_sides[SIDE_TOP]);
         xfwmPixmapFree (&frame_pix.pm_sides[SIDE_BOTTOM]);
         xfwmPixmapFree (&frame_pix.pm_sides[SIDE_LEFT]);
         xfwmPixmapFree (&frame_pix.pm_sides[SIDE_RIGHT]);
@@ -1124,7 +1156,7 @@ frameDraw (Client * c, gboolean clear_all)
         {
             xfwmWindowHide (&c->title);
         }
-        for (i = 0; i < 3; i++)
+        for (i = 0; i < 4; i++)
         {
             if (MYWINDOW_XWINDOW (c->sides[i]) && xfwmWindowVisible (&c->sides[i]))
             {
@@ -1138,7 +1170,7 @@ frameDraw (Client * c, gboolean clear_all)
                 xfwmWindowHide (&c->corners[i]);
             }
         }
-        for (i = 0; i < BUTTON_LAST; i++)
+        for (i = 0; i < BUTTON_COUNT; i++)
         {
             if (MYWINDOW_XWINDOW (c->buttons[i]) && xfwmWindowVisible (&c->buttons[i]))
             {
@@ -1153,13 +1185,13 @@ static gboolean
 update_frame_idle_cb (gpointer data)
 {
     Client *c;
-    
+
     TRACE ("entering update_frame_idle_cb");
 
     c = (Client *) data;
     g_return_val_if_fail (c, FALSE);
 
-    frameDraw (c, TRUE);
+    frameDrawWin (c);
     c->frame_timeout_id = 0;
 
     return FALSE;
@@ -1180,15 +1212,39 @@ frameClearQueueDraw (Client * c)
 }
 
 void
-frameQueueDraw (Client * c)
+frameDraw (Client * c, gboolean clear_all)
+{
+    g_return_if_fail (c);
+
+    TRACE ("entering frameDraw for \"%s\" (0x%lx)", c->name, c->window);
+
+    if (clear_all)
+    {
+        FLAG_SET (c->xfwm_flags, XFWM_FLAG_NEEDS_REDRAW);
+    }
+    frameDrawWin (c);
+}
+
+void
+frameQueueDraw (Client * c, gboolean clear_all)
 {
     g_return_if_fail (c);
 
     TRACE ("entering frameQueueDraw for \"%s\" (0x%lx)", c->name, c->window);
 
+    /* Reschedule update */
+    if (c->frame_timeout_id)
+    {
+        frameClearQueueDraw (c);
+    }
+    if (clear_all)
+    {
+        FLAG_SET (c->xfwm_flags, XFWM_FLAG_NEEDS_REDRAW);
+    }
+    /* Otherwise leave previous schedule */
     if (c->frame_timeout_id == 0)
     {
-        c->frame_timeout_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, 
+        c->frame_timeout_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
                                               update_frame_idle_cb, c, NULL);
     }
 }
