@@ -1,4 +1,4 @@
-/*      $Id: placement.c 28239 2008-10-14 08:04:11Z olivier $
+/*      $Id: placement.c 29029 2008-12-16 22:16:01Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -38,15 +38,10 @@
 #include "frame.h"
 #include "netwm.h"
 
-static unsigned long overlapX (int x0, int x1, int tx0, int tx1);
-static unsigned long overlapY (int y0, int y1, int ty0, int ty1);
-static unsigned long overlap (int x0, int y0, int x1, int y1,
-                              int tx0, int ty0, int tx1, int ty1);
-
 /* Compute rectangle overlap area */
 
 static unsigned long
-overlapX (int x0, int x1, int tx0, int tx1)
+segment_overlap (int x0, int x1, int tx0, int tx1)
 {
     if (tx0 > x0)
     {
@@ -63,29 +58,12 @@ overlapX (int x0, int x1, int tx0, int tx1)
     return (x1 - x0);
 }
 
-static unsigned long
-overlapY (int y0, int y1, int ty0, int ty1)
-{
-    if (ty0 > y0)
-    {
-        y0 = ty0;
-    }
-    if (ty1 < y1)
-    {
-        y1 = ty1;
-    }
-    if (y1 <= y0)
-    {
-        return 0;
-    }
-    return (y1 - y0);
-}
-
 static unsigned long
 overlap (int x0, int y0, int x1, int y1, int tx0, int ty0, int tx1, int ty1)
 {
     /* Compute overlapping box */
-    return (overlapX (x0, x1, tx0, tx1) * overlapY (y0, y1, ty0, ty1));
+    return (segment_overlap (x0, x1, tx0, tx1)
+            * segment_overlap (y0, y1, ty0, ty1));
 }
 
 static unsigned long
@@ -275,10 +253,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_RIGHT_START_Y], c2->struts[STRUTS_RIGHT_END_Y]))
                 {
-                    if (overlapX (frame_x, frame_x + frame_width,
+                    if (segment_overlap (frame_x, frame_x + frame_width,
                                   screen_width - c2->struts[STRUTS_RIGHT],
                                   screen_width))
                     {
@@ -289,10 +267,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_BOTTOM_START_X], c2->struts[STRUTS_BOTTOM_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_height,
+                    if (segment_overlap (frame_y, frame_y + frame_height,
                                   screen_height - c2->struts[STRUTS_BOTTOM],
                                   screen_height))
                     {
@@ -337,10 +315,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_LEFT_START_Y], c2->struts[STRUTS_LEFT_END_Y]))
                 {
-                    if (overlapX (frame_x, frame_x + frame_width,
+                    if (segment_overlap (frame_x, frame_x + frame_width,
                                   0, c2->struts[STRUTS_LEFT]))
                     {
                         c->x = c2->struts[STRUTS_LEFT] + frame_left;
@@ -350,12 +328,12 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Top */
-                if (overlapX (frame_x,
+                if (segment_overlap (frame_x,
                               frame_x + frame_width,
                               c2->struts[STRUTS_TOP_START_X],
                               c2->struts[STRUTS_TOP_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_height,
+                    if (segment_overlap (frame_y, frame_y + frame_height,
                                   0, c2->struts[STRUTS_TOP]))
                     {
                         c->y = c2->struts[STRUTS_TOP] + frame_top;
@@ -406,7 +384,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_RIGHT_START_Y], c2->struts[STRUTS_RIGHT_END_Y]))
                 {
                     if (frame_x >= screen_width - c2->struts[STRUTS_RIGHT] - min_visible)
@@ -418,7 +396,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_LEFT_START_Y], c2->struts[STRUTS_LEFT_END_Y]))
                 {
                     if (frame_x + frame_width <= c2->struts[STRUTS_LEFT] + min_visible)
@@ -430,7 +408,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_BOTTOM_START_X], c2->struts[STRUTS_BOTTOM_END_X]))
                 {
                     if (frame_y >= screen_height - c2->struts[STRUTS_BOTTOM] - min_visible)
@@ -442,10 +420,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Top */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_TOP_START_X], c2->struts[STRUTS_TOP_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_visible, 0, c2->struts[STRUTS_TOP]))
+                    if (segment_overlap (frame_y, frame_y + frame_visible, 0, c2->struts[STRUTS_TOP]))
                     {
                         c->y = c2->struts[STRUTS_TOP] + frame_top;
                         frame_y = frameY (c);
@@ -586,11 +564,11 @@ smartPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
     best_x = full_x + frameLeft (c);
     best_y = full_y + frameTop (c);
 
-    for (test_y = full_y + frameTop (c); test_y <= ymax;
-        test_y += 8)
+    test_y = full_y + frameTop (c);
+    do
     {
-        for (test_x = full_x + frameLeft (c);
-            test_x <= xmax; test_x += 8)
+        test_x = full_x + frameLeft (c);
+        do
         {
             gfloat count_overlaps = 0.0;
             TRACE ("analyzing %i clients", screen_info->client_count);
@@ -628,8 +606,13 @@ smartPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
             {
                 first = FALSE;
             }
+            test_x += 8;
         }
+        while (test_x <= xmax);
+        test_y += 8;
     }
+    while (test_y <= ymax);
+
     c->x = best_x;
     c->y = best_y;
 }
@@ -761,3 +744,227 @@ clientInitPosition (Client * c)
         clientAutoMaximize (c, full_w, full_h);
     }
 }
+
+
+void
+clientFill (Client * c, int fill_type)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    Client *east_neighbour;
+    Client *west_neighbour;
+    Client *north_neighbour;
+    Client *south_neighbour;
+    Client *c2;
+    GdkRectangle rect;
+    XWindowChanges wc;
+    unsigned short mask;
+    int i, cx, cy, full_x, full_y, full_w, full_h;
+    int tmp_x, tmp_y, tmp_w, tmp_h;
+    gint monitor_nbr;
+
+    g_return_if_fail (c != NULL);
+
+    if (!CLIENT_CAN_FILL_WINDOW (c))
+    {
+        return;
+    }
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+    mask = 0;
+    east_neighbour = NULL;
+    west_neighbour = NULL;
+    north_neighbour = NULL;
+    south_neighbour = NULL;
+
+    for (c2 = screen_info->clients, i = 0; i < screen_info->client_count; c2 = c2->next, i++)
+    {
+
+        /* Filter out all windows which are not visible, or not on the same layer
+         * as well as the client window itself
+         */
+        if ((c != c2) && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE) && (c2->win_layer == c->win_layer))
+        {
+            /* Fill horizontally */
+            if (fill_type & CLIENT_FILL_HORIZ)
+            {
+                /*
+                 * check if the neigbour client (c2) is located
+                 * east or west of our client.
+                 */
+                if (segment_overlap (frameY(c), frameY(c) + frameHeight(c), frameY(c2), frameY(c2) + frameHeight(c2)))
+                {
+                    if ((frameX(c2) + frameWidth(c2)) < frameX(c))
+                    {
+                        if (east_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the east neighbour already found
+                             */
+                            if ((frameX(east_neighbour) + frameWidth(east_neighbour)) < (frameX(c2) + frameWidth(c2)))
+                            {
+                                east_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            east_neighbour = c2;
+                        }
+                    }
+                    if ((frameX(c) + frameWidth(c)) < frameX(c2))
+                    {
+                        /* Check if c2 is closer to the client
+                         * then the west neighbour already found
+                         */
+                        if (west_neighbour)
+                        {
+                            if (frameX(c2) < frameX(west_neighbour))
+                            {
+                                west_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            west_neighbour = c2;
+                        }
+                    }
+                }
+            }
+
+            /* Fill vertically */
+            if (fill_type & CLIENT_FILL_VERT)
+            {
+                /* check if the neigbour client (c2) is located
+                 * north or south of our client.
+                 */
+                if (segment_overlap (frameX(c), frameX(c) + frameWidth(c), frameX(c2), frameX(c2) + frameWidth(c2)))
+                {
+                    if ((frameY(c2) + frameHeight(c2)) < frameY(c))
+                    {
+                        if (north_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the north neighbour already found
+                             */
+                            if ((frameY(north_neighbour) + frameHeight(north_neighbour)) < (frameY(c2) + frameHeight(c2)))
+                            {
+                                north_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            north_neighbour = c2;
+                        }
+                    }
+                    if ((frameY(c) + frameHeight(c)) < frameY(c2))
+                    {
+                        if (south_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the south neighbour already found
+                             */
+                            if (frameY(c2) < frameY(south_neighbour))
+                            {
+                                south_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            south_neighbour = c2;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    /* Compute the largest size available, based on struts, margins and Xinerama layout */
+    tmp_x = frameX (c);
+    tmp_y = frameY (c);
+    tmp_h = frameHeight (c);
+    tmp_w = frameWidth (c);
+
+    cx = tmp_x + (tmp_w / 2);
+    cy = tmp_y + (tmp_h / 2);
+
+    monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
+    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+
+    full_x = MAX (screen_info->params->xfwm_margins[STRUTS_LEFT], rect.x);
+    full_y = MAX (screen_info->params->xfwm_margins[STRUTS_TOP], rect.y);
+    full_w = MIN (screen_info->width - screen_info->params->xfwm_margins[STRUTS_RIGHT],
+                  rect.x + rect.width) - full_x;
+    full_h = MIN (screen_info->height - screen_info->params->xfwm_margins[STRUTS_BOTTOM],
+                  rect.y + rect.height) - full_y;
+
+    if ((fill_type & CLIENT_FILL) == CLIENT_FILL)
+    {
+        mask = CWX | CWY | CWHeight | CWWidth;
+        /* Adjust size to the largest size available, not covering struts */
+        clientMaxSpace (screen_info, &full_x, &full_y, &full_w, &full_h);
+    }
+    else if (fill_type & CLIENT_FILL_VERT)
+    {
+        mask = CWY | CWHeight;
+        /* Adjust size to the tallest size available, for the current horizontal position/width */
+        clientMaxSpace (screen_info, &tmp_x, &full_y, &tmp_w, &full_h);
+    }
+    else if (fill_type & CLIENT_FILL_HORIZ)
+    {
+        mask = CWX | CWWidth;
+        /* Adjust size to the widest size available, for the current vertical position/height */
+        clientMaxSpace (screen_info, &full_x, &tmp_y, &full_w, &tmp_h);
+    }
+
+    /* If there are neighbours, resize to their borders.
+     * If not, resize to the largest size available that you just have computed.
+     */
+
+    wc.x = frameLeft(c);
+    if (east_neighbour)
+    {
+        wc.x += MAX (frameX(east_neighbour) + frameWidth(east_neighbour), full_x);
+    }
+    else
+    {
+        wc.x += full_x;
+    }
+
+    wc.width = full_x - frameRight(c) - wc.x;
+    if (west_neighbour)
+    {
+        wc.width += MIN (frameX(west_neighbour), full_w);
+    }
+    else
+    {
+        wc.width += full_w;
+    }
+
+    wc.y = frameTop(c);
+    if (north_neighbour)
+    {
+        wc.y += MAX (frameY(north_neighbour) + frameHeight(north_neighbour), full_y);
+    }
+    else
+    {
+        wc.y += full_y;
+    }
+
+    wc.height = full_y - frameBottom(c) - wc.y;
+    if (south_neighbour)
+    {
+        wc.height += MIN (frameY(south_neighbour), full_h);
+    }
+    else
+    {
+        wc.height += full_h;
+    }
+
+    TRACE ("Fill size request: (%d,%d) %dx%d", wc.x, wc.y, wc.width, wc.height);
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
+    {
+        clientConfigure(c, &wc, mask, NO_CFG_FLAG);
+    }
+}
+
