@@ -1,4 +1,4 @@
-/*      $Id: session.c 23493 2006-10-23 20:05:28Z olivier $
+/*      $Id: session.c 28284 2008-10-17 13:10:03Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
  */
 
 /* Initially inspired by xfwm, fvwm2, enlightment and twm implementations */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xatom.h>
@@ -284,7 +286,7 @@ sessionSaveScreen (ScreenInfo *screen_info, FILE *f)
         {
             continue;
         }
-        
+
         if (c->client_leader != None)
         {
             getWindowRole (display_info, c->window, &window_role);
@@ -766,6 +768,7 @@ sessionDie (gpointer data)
     DisplayInfo *display_info;
 
     display_info = (DisplayInfo *) data;
+    client_session_set_restart_style(display_info->session, SESSION_RESTART_IF_RUNNING);    
     display_info->quit = TRUE;
     gtk_main_quit ();
 }
@@ -776,7 +779,7 @@ sessionStart (int argc, char **argv, DisplayInfo *display_info)
     SessionClient *session;
 
     display_info->session = client_session_new (argc, argv, (gpointer) display_info,
-                                                SESSION_RESTART_IF_RUNNING, 20);
+                                                SESSION_RESTART_IMMEDIATELY, 20);
     session = display_info->session;
     session->data = (gpointer) display_info;
     session->save_phase_2 = sessionSavePhase2;
