@@ -1,4 +1,4 @@
-/*      $Id: settings.c 27421 2008-08-04 20:28:44Z olivier $
+/*      $Id: settings.c 28758 2008-11-11 18:18:01Z jannis $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -17,7 +17,9 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan,
+                       2008 Jannis Pohlmann
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +33,7 @@
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
 #include <xfconf/xfconf.h>
+#include <libxfce4kbd-private/xfce-shortcuts-provider.h>
 
 #include "screen.h"
 #include "hints.h"
@@ -41,13 +44,8 @@
 #include "compositor.h"
 #include "ui_style.h"
 
-#define CHANNEL1                "xfwm4"
-#define CHANNEL2                "xfwm4_keys"
-
-#define DEFAULT_KEYTHEME        "Default"
-#define KEYTHEMERC              "keythemerc"
+#define CHANNEL_XFWM            "xfwm4"
 #define THEMERC                 "themerc"
-
 #define XPM_COLOR_SYMBOL_SIZE   22
 #define XFWM4_SETTINGS_COUNT    66
 
@@ -61,16 +59,30 @@ static void              loadRcData           (ScreenInfo *,
                                                Settings *);
 static void              loadTheme            (ScreenInfo *,
                                                Settings *);
-static gboolean          loadKeyBindings      (ScreenInfo *,
-                                               Settings *);
+static void              loadKeyBindings      (ScreenInfo *);
 static void              unloadTheme          (ScreenInfo *);
+static void              unloadKeyBindings    (ScreenInfo *);
 static void              unloadSettings       (ScreenInfo *);
 static gboolean          reloadScreenSettings (ScreenInfo *,
                                                int);
+static void              parseShortcut        (ScreenInfo *,
+                                               int,
+                                               const gchar *,
+                                               GList *);
+static const gchar      *getShortcut          (const gchar *,
+                                               GList *);
 static void              cb_xfwm4_channel_property_changed (XfconfChannel *,
                                                            const gchar *,
                                                            const GValue *,
                                                            ScreenInfo *);
+static void              cb_keys_changed      (GdkKeymap *,
+                                               ScreenInfo *);
+static void              cb_shortcut_added    (XfceShortcutsProvider *,
+                                               const gchar *,
+                                               ScreenInfo *);
+static void              cb_shortcut_removed  (XfceShortcutsProvider *,
+                                               const gchar *,
+                                               ScreenInfo *);
 
 static void
 update_grabs (ScreenInfo *screen_info)
@@ -129,6 +141,8 @@ set_settings_margin (ScreenInfo *screen_info, int idx, int value)
 static void
 set_easy_click (ScreenInfo *screen_info, const char *modifier)
 {
+    gchar *modstr;
+
     g_return_if_fail (screen_info != NULL);
     g_return_if_fail (modifier != NULL);
 
@@ -138,7 +152,9 @@ set_easy_click (ScreenInfo *screen_info, const char *modifier)
     }
     else
     {
-        screen_info->params->easy_click = getModifierMap (modifier);
+        modstr = g_strdup_printf ("<%s>", modifier);
+        screen_info->params->easy_click = getModifierMap (modstr);
+        g_free (modstr);
     }
 }
 
@@ -182,30 +198,12 @@ static void
 loadRcData (ScreenInfo *screen_info, Settings *rc)
 {
     gchar *homedir;
-    const gchar *keythemevalue;
-    gchar *keytheme;
-    gchar *system_keytheme;
 
     if (!parseRc ("defaults", PACKAGE_DATADIR, rc))
     {
         g_warning ("Missing defaults file");
         exit (1);
     }
-    keythemevalue = getStringValue ("keytheme", rc);
-    if (keythemevalue)
-    {
-        system_keytheme = getSystemThemeDir ();
-        parseRc (KEYTHEMERC, system_keytheme, rc);
-
-        keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
-        if (keytheme)
-        {
-            /* If there is a custom key theme, merge it with system defaults */
-            parseRc (KEYTHEMERC, keytheme, rc);
-            g_free (keytheme);
-        }
-        g_free (system_keytheme);
-    }
     homedir = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
                                            "xfce4" G_DIR_SEPARATOR_S "xfwm4",
                                            FALSE);
@@ -535,96 +533,75 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     g_free (theme);
 }
 
-static gboolean
-loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
+static void
+loadKeyBindings (ScreenInfo *screen_info)
 {
+    GList *shortcuts;
     gchar keyname[30];
     Display *dpy;
-    gchar *keytheme;
-    const gchar *keythemevalue;
     guint i;
 
     dpy = myScreenGetXDisplay (screen_info);
-    /*
-       Load defaults keytheme so that even if there are
-       missing shortcuts in an older user defined key theme
-       the missing keys will be taken from the default
-     */
-    keytheme = getThemeDir (DEFAULT_KEYTHEME, KEYTHEMERC);
-    parseRc (KEYTHEMERC, keytheme, rc);
-    g_free (keytheme);
 
-    keythemevalue = getStringValue ("keytheme", rc);
-    if (keythemevalue)
-    {
-        keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
-        if (!parseRc (KEYTHEMERC, keytheme, rc))
-        {
-            g_warning ("Specified key theme \"%s\" missing, using default", keythemevalue);
-        }
-        g_free (keytheme);
+    shortcuts = xfce_shortcuts_provider_get_shortcuts (screen_info->shortcuts_provider);
 
-        if (!checkRc (rc))
-        {
-            g_warning ("Missing values in defaults file");
-            return FALSE;
-        }
-    }
-
-    parseKeyString (dpy, &screen_info->params->keys[KEY_CANCEL], getStringValue ("cancel_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_DOWN], getStringValue ("down_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_LEFT], getStringValue ("left_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_RIGHT], getStringValue ("right_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_UP], getStringValue ("up_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], getStringValue ("add_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_ADD_ADJACENT_WORKSPACE], getStringValue ("add_adjacent_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_CLOSE_WINDOW], getStringValue ("close_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], getStringValue ("cycle_windows_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], getStringValue ("del_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_DEL_ACTIVE_WORKSPACE], getStringValue ("del_active_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], getStringValue ("down_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_FILL_HORIZ], getStringValue ("fill_horiz_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_FILL_VERT], getStringValue ("fill_vert_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_FILL_WINDOW], getStringValue ("fill_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_HIDE_WINDOW], getStringValue ("hide_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], getStringValue ("left_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_LOWER_WINDOW], getStringValue ("lower_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE], getStringValue ("move_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_HORIZ], getStringValue ("maximize_horiz_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_VERT], getStringValue ("maximize_vert_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_WINDOW], getStringValue ("maximize_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_DOWN_WORKSPACE], getStringValue ("move_window_down_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_LEFT_WORKSPACE], getStringValue ("move_window_left_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_NEXT_WORKSPACE], getStringValue ("move_window_next_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_PREV_WORKSPACE], getStringValue ("move_window_prev_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_RIGHT_WORKSPACE], getStringValue ("move_window_right_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_UP_WORKSPACE], getStringValue ("move_window_up_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], getStringValue ("next_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_POPUP_MENU], getStringValue ("popup_menu_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], getStringValue ("prev_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_RAISE_WINDOW], getStringValue ("raise_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_RESIZE], getStringValue ("resize_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], getStringValue ("right_workspace_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_SHADE_WINDOW], getStringValue ("shade_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_SHOW_DESKTOP], getStringValue("show_desktop_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_STICK_WINDOW], getStringValue ("stick_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_TOGGLE_ABOVE], getStringValue ("above_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_TOGGLE_FULLSCREEN], getStringValue ("fullscreen_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], getStringValue ("up_workspace_key", rc));
+    parseShortcut (screen_info, KEY_CANCEL, "cancel_key", shortcuts);
+    parseShortcut (screen_info, KEY_DOWN, "down_key", shortcuts);
+    parseShortcut (screen_info, KEY_LEFT, "left_key", shortcuts);
+    parseShortcut (screen_info, KEY_RIGHT, "right_key", shortcuts);
+    parseShortcut (screen_info, KEY_UP, "up_key", shortcuts);
+    parseShortcut (screen_info, KEY_ADD_WORKSPACE, "add_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_ADD_ADJACENT_WORKSPACE, "add_adjacent_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_CLOSE_WINDOW, "close_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_CYCLE_WINDOWS, "cycle_windows_key", shortcuts);
+    parseShortcut (screen_info, KEY_CYCLE_REVERSE_WINDOWS, "cycle_reverse_windows_key", shortcuts);
+    parseShortcut (screen_info, KEY_DEL_WORKSPACE, "del_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_DEL_ACTIVE_WORKSPACE, "del_active_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_DOWN_WORKSPACE, "down_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_FILL_HORIZ, "fill_horiz_key", shortcuts);
+    parseShortcut (screen_info, KEY_FILL_VERT, "fill_vert_key", shortcuts);
+    parseShortcut (screen_info, KEY_FILL_WINDOW, "fill_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_HIDE_WINDOW, "hide_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_LEFT_WORKSPACE, "left_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_LOWER_WINDOW, "lower_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE, "move_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_MAXIMIZE_HORIZ, "maximize_horiz_key", shortcuts);
+    parseShortcut (screen_info, KEY_MAXIMIZE_VERT, "maximize_vert_key", shortcuts);
+    parseShortcut (screen_info, KEY_MAXIMIZE_WINDOW, "maximize_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_DOWN_WORKSPACE, "move_window_down_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_LEFT_WORKSPACE, "move_window_left_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_NEXT_WORKSPACE, "move_window_next_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_PREV_WORKSPACE, "move_window_prev_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_RIGHT_WORKSPACE, "move_window_right_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_MOVE_UP_WORKSPACE, "move_window_up_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_NEXT_WORKSPACE, "next_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_POPUP_MENU, "popup_menu_key", shortcuts);
+    parseShortcut (screen_info, KEY_PREV_WORKSPACE, "prev_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_RAISE_WINDOW, "raise_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_RESIZE, "resize_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_RIGHT_WORKSPACE, "right_workspace_key", shortcuts);
+    parseShortcut (screen_info, KEY_SHADE_WINDOW, "shade_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_SHOW_DESKTOP, "show_desktop_key", shortcuts);
+    parseShortcut (screen_info, KEY_STICK_WINDOW, "stick_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_TOGGLE_ABOVE, "above_key", shortcuts);
+    parseShortcut (screen_info, KEY_TOGGLE_FULLSCREEN, "fullscreen_key", shortcuts);
+    parseShortcut (screen_info, KEY_UP_WORKSPACE, "up_workspace_key", shortcuts);
 
     for (i = 0; i < 12; i++)
     {
         g_snprintf(keyname, sizeof (keyname), "move_window_workspace_%d_key", i + 1);
-        parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_WORKSPACE_1 + i], getStringValue (keyname, rc));
+        parseShortcut (screen_info, KEY_MOVE_WORKSPACE_1 + i, keyname, shortcuts);
 
         g_snprintf(keyname, sizeof (keyname), "workspace_%d_key", i + 1);
-        parseKeyString (dpy, &screen_info->params->keys[KEY_WORKSPACE_1 + i], getStringValue (keyname, rc));
+        parseShortcut (screen_info, KEY_WORKSPACE_1 + i, keyname, shortcuts);
     }
 
+    xfce_shortcuts_free (shortcuts);
+
     myScreenUngrabKeys (screen_info);
     myScreenGrabKeys (screen_info);
 
-    return TRUE;
+    return;
 }
 
 gboolean
@@ -665,6 +642,7 @@ loadSettings (ScreenInfo *screen_info)
         {"button_spacing", NULL, G_TYPE_INT, TRUE},
         {"click_to_focus", NULL, G_TYPE_BOOLEAN, TRUE},
         {"focus_delay", NULL, G_TYPE_INT, TRUE},
+        {"cycle_draw_frame", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_hidden", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_minimum", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -676,7 +654,6 @@ loadSettings (ScreenInfo *screen_info)
         {"frame_opacity", NULL, G_TYPE_INT, TRUE},
         {"full_width_title", NULL, G_TYPE_BOOLEAN, TRUE},
         {"inactive_opacity", NULL, G_TYPE_INT, TRUE},
-        {"keytheme", NULL, G_TYPE_STRING, TRUE},
         {"margin_bottom", NULL, G_TYPE_INT, FALSE},
         {"margin_left", NULL, G_TYPE_INT, FALSE},
         {"margin_right", NULL, G_TYPE_INT, FALSE},
@@ -691,6 +668,7 @@ loadSettings (ScreenInfo *screen_info)
         {"raise_on_click", NULL, G_TYPE_BOOLEAN, TRUE},
         {"raise_on_focus", NULL, G_TYPE_BOOLEAN, TRUE},
         {"raise_with_any_button", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"repeat_urgent_blink", NULL, G_TYPE_BOOLEAN, TRUE},
         {"resize_opacity", NULL, G_TYPE_INT, TRUE},
         {"restore_on_move", NULL, G_TYPE_BOOLEAN, TRUE},
         {"scroll_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -698,6 +676,7 @@ loadSettings (ScreenInfo *screen_info)
         {"shadow_delta_width", NULL, G_TYPE_INT, TRUE},
         {"shadow_delta_x", NULL, G_TYPE_INT, TRUE},
         {"shadow_delta_y", NULL, G_TYPE_INT, TRUE},
+        {"shadow_opacity", NULL, G_TYPE_INT, TRUE},
         {"show_app_icon", NULL, G_TYPE_BOOLEAN, TRUE},
         {"show_dock_shadow", NULL, G_TYPE_BOOLEAN, TRUE},
         {"show_frame_shadow", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -716,6 +695,7 @@ loadSettings (ScreenInfo *screen_info)
         {"title_vertical_offset_inactive", NULL, G_TYPE_INT, TRUE},
         {"toggle_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
         {"unredirect_overlays", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"urgent_blink", NULL, G_TYPE_BOOLEAN, TRUE},
         {"use_compositing", NULL, G_TYPE_BOOLEAN, TRUE},
         {"workspace_count", NULL, G_TYPE_INT, TRUE},
         {"wrap_cycle", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -730,6 +710,7 @@ loadSettings (ScreenInfo *screen_info)
         {"cancel_key", NULL, G_TYPE_STRING, TRUE},
         {"close_window_key", NULL, G_TYPE_STRING, TRUE},
         {"cycle_windows_key", NULL, G_TYPE_STRING, TRUE},
+        {"cycle_reverse_windows_key", NULL, G_TYPE_STRING, TRUE},
         {"del_active_workspace_key", NULL, G_TYPE_STRING, TRUE},
         {"del_workspace_key", NULL, G_TYPE_STRING, TRUE},
         {"down_key", NULL, G_TYPE_STRING, TRUE},
@@ -799,11 +780,7 @@ loadSettings (ScreenInfo *screen_info)
     loadTheme (screen_info, rc);
     update_grabs (screen_info);
 
-    if (!loadKeyBindings (screen_info, rc))
-    {
-        freeRc (rc);
-        return FALSE;
-    }
+    loadKeyBindings (screen_info);
 
     screen_info->params->borderless_maximize =
         getBoolValue ("borderless_maximize", rc);
@@ -815,6 +792,8 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("click_to_focus", rc);
     screen_info->params->cycle_minimum =
         getBoolValue ("cycle_minimum", rc);
+    screen_info->params->cycle_draw_frame =
+        getBoolValue ("cycle_draw_frame", rc);
     screen_info->params->cycle_hidden =
         getBoolValue ("cycle_hidden", rc);
     screen_info->params->cycle_workspaces =
@@ -835,20 +814,26 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("raise_on_click", rc);
     screen_info->params->raise_with_any_button =
         getBoolValue ("raise_with_any_button", rc);
+    screen_info->params->repeat_urgent_blink =
+        getBoolValue ("repeat_urgent_blink", rc);
+    screen_info->params->urgent_blink =
+        getBoolValue ("urgent_blink", rc);
     screen_info->params->restore_on_move =
         getBoolValue ("restore_on_move", rc);
     screen_info->params->frame_opacity =
-        getIntValue ("frame_opacity", rc);
+        CLAMP (getIntValue ("frame_opacity", rc), 0, 100);
     screen_info->params->inactive_opacity =
-        getIntValue ("inactive_opacity", rc);
+        CLAMP (getIntValue ("inactive_opacity", rc), 0, 100);
     screen_info->params->move_opacity =
-        getIntValue ("move_opacity", rc);
+        CLAMP (getIntValue ("move_opacity", rc), 0, 100);
     screen_info->params->resize_opacity =
-        getIntValue ("resize_opacity", rc);
+        CLAMP (getIntValue ("resize_opacity", rc), 0, 100);
     screen_info->params->popup_opacity =
-        getIntValue ("popup_opacity", rc);
+        CLAMP (getIntValue ("popup_opacity", rc), 0, 100);
     screen_info->params->placement_ratio =
-        getIntValue ("placement_ratio", rc);
+        CLAMP (getIntValue ("placement_ratio", rc), 0, 100);
+    screen_info->params->shadow_opacity =
+        CLAMP (getIntValue ("shadow_opacity", rc), 0, 100);
     screen_info->params->show_app_icon =
         getBoolValue ("show_app_icon", rc);
     screen_info->params->show_dock_shadow =
@@ -904,11 +889,7 @@ loadSettings (ScreenInfo *screen_info)
     if (screen_info->workspace_count < 0)
     {
         gint workspace_count;
-        workspace_count = getIntValue ("workspace_count", rc);
-        if (workspace_count < 0)
-        {
-            workspace_count = 0;
-        }
+        workspace_count = MAX (getIntValue ("workspace_count", rc), 0);
         workspaceSetCount (screen_info, workspace_count);
     }
 
@@ -973,6 +954,20 @@ unloadTheme (ScreenInfo *screen_info)
     }
 }
 
+
+static void
+unloadKeyBindings (ScreenInfo *screen_info)
+{
+    int i;
+
+    g_return_if_fail (screen_info);
+
+    for (i = 0; i < KEY_COUNT; ++i)
+    {
+        g_free (screen_info->params->keys[i].internal_name);
+    }
+}
+
 static void
 unloadSettings (ScreenInfo *screen_info)
 {
@@ -981,6 +976,7 @@ unloadSettings (ScreenInfo *screen_info)
     TRACE ("entering unloadSettings");
 
     unloadTheme (screen_info);
+    unloadKeyBindings (screen_info);
 }
 
 static gboolean
@@ -1030,6 +1026,7 @@ reloadSettings (DisplayInfo *display_info, int mask)
 gboolean
 initSettings (ScreenInfo *screen_info)
 {
+    GdkKeymap *keymap;
     DisplayInfo *display_info;
     char **names;
     long val;
@@ -1045,17 +1042,24 @@ initSettings (ScreenInfo *screen_info)
         return FALSE;
     }
 
-
     display_info = screen_info->display_info;
     names = NULL;
     val = 0;
     i = 0;
 
-    screen_info->xfwm4_channel = xfconf_channel_new(CHANNEL1);
-    screen_info->keys_channel = xfconf_channel_new(CHANNEL2);
+    screen_info->xfwm4_channel = xfconf_channel_new(CHANNEL_XFWM);
+    g_signal_connect (screen_info->xfwm4_channel, "property-changed",
+                      G_CALLBACK (cb_xfwm4_channel_property_changed), screen_info);
 
-    g_signal_connect (G_OBJECT(screen_info->xfwm4_channel), "property-changed", (GCallback)cb_xfwm4_channel_property_changed, screen_info);
+    keymap = gdk_keymap_get_default ();
+    g_signal_connect (keymap, "keys-changed",
+                      G_CALLBACK (cb_keys_changed), screen_info);
 
+    screen_info->shortcuts_provider = xfce_shortcuts_provider_new ("xfwm4");
+    g_signal_connect (screen_info->shortcuts_provider, "shortcut-added",
+                      G_CALLBACK (cb_shortcut_added), screen_info);
+    g_signal_connect (screen_info->shortcuts_provider, "shortcut-removed",
+                      G_CALLBACK (cb_shortcut_removed), screen_info);
 
     if (!loadSettings (screen_info))
     {
@@ -1123,11 +1127,7 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     reloadScreenSettings (screen_info, UPDATE_FRAME | UPDATE_CACHE);
                 }
-                else if (!strcmp (name, "keytheme"))
-                {
-                    reloadScreenSettings (screen_info, NO_UPDATE_FLAG);
-                }
-                else if (!strcmp (name, "easy_click"))
+                 else if (!strcmp (name, "easy_click"))
                 {
                     reloadScreenSettings (screen_info, UPDATE_BUTTON_GRABS);
                 }
@@ -1139,6 +1139,11 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     set_placement_mode (screen_info, g_value_get_string (value));
                 }
+                else if ((!strcmp (name, "title_shadow_active"))
+                      || (!strcmp (name, "title_shadow_inactive")))
+                {
+                    /* These properties are not configurable via xfconf */
+                }
                 else
                 {
                     g_warning("The property '%s' is not supported", property_name);
@@ -1209,6 +1214,21 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->placement_ratio = g_value_get_int (value);
                 }
+                else if ((!strcmp (name, "button_offset"))
+                      || (!strcmp (name, "button_spacing"))
+                      || (!strcmp (name, "dbl_click_time"))
+                      || (!strcmp (name, "maximized_offset"))
+                      || (!strcmp (name, "shadow_delta_height"))
+                      || (!strcmp (name, "shadow_delta_width"))
+                      || (!strcmp (name, "shadow_delta_x"))
+                      || (!strcmp (name, "shadow_delta_y"))
+                      || (!strcmp (name, "shadow_opacity"))
+                      || (!strcmp (name, "title_horizontal_offset"))
+                      || (!strcmp (name, "title_vertical_offset_active"))
+                      || (!strcmp (name, "title_vertical_offset_inactive")))
+                {
+                    /* These properties are not configurable via xfconf */
+                }
                 else
                 {
                     g_warning("The property '%s' is not supported", property_name);
@@ -1218,7 +1238,6 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 if (!strcmp (name, "box_move"))
                 {
                     screen_info->params->box_move = g_value_get_boolean (value);
-
                 }
                 else if (!strcmp (name, "box_resize"))
                 {
@@ -1242,6 +1261,14 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                     screen_info->params->raise_on_click = g_value_get_boolean (value);
                     update_grabs (screen_info);
                 }
+                else if (!strcmp (name, "repeat_urgent_blink"))
+                {
+                    screen_info->params->repeat_urgent_blink = g_value_get_boolean (value);
+                }
+                else if (!strcmp (name, "urgent_blink"))
+                {
+                    screen_info->params->urgent_blink = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "snap_to_border"))
                 {
                     screen_info->params->snap_to_border = g_value_get_boolean (value);
@@ -1253,6 +1280,7 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 else if (!strcmp (name, "wrap_workspaces"))
                 {
                     screen_info->params->wrap_workspaces = g_value_get_boolean (value);
+                    placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
                 }
                 else if (!strcmp (name, "wrap_windows"))
                 {
@@ -1267,6 +1295,10 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->cycle_minimum = g_value_get_boolean (value);
                 }
+                else if (!strcmp (name, "cycle_draw_frame"))
+                {
+                    screen_info->params->cycle_draw_frame = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "cycle_hidden"))
                 {
                     screen_info->params->cycle_hidden = g_value_get_boolean (value);
@@ -1337,14 +1369,138 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->wrap_cycle = g_value_get_boolean (value);
                 }
+                else if ((!strcmp (name, "full_width_title"))
+                      || (!strcmp (name, "show_app_icon")))
+                {
+                    /* These properties are not configurable via xfconf */
+                }
                 else
                 {
                     g_warning("The property '%s' is not supported", property_name);
                 }
                 break;
             default:
-                g_warning("The type of property '%s' is not supported", property_name);
+                if (!strcmp (name, "workspace_names"))
+                {
+                    /* These properties are not configurable via xfconf */
+                }
+                else
+                {
+                    g_warning("The property '%s' is not supported", property_name);
+                }
                 break;
         }
     }
 }
+
+static void
+cb_keys_changed (GdkKeymap *keymap, ScreenInfo *screen_info)
+{
+    initModifiers (myScreenGetXDisplay (screen_info));
+    reloadSettings (screen_info->display_info, UPDATE_BUTTON_GRABS);
+}
+
+static void
+cb_shortcut_added (XfceShortcutsProvider *provider, const gchar *shortcut,
+                   ScreenInfo *screen_info)
+{
+    XfceShortcut *sc;
+    Display *dpy;
+    int i;
+
+    g_return_if_fail (XFCE_IS_SHORTCUTS_PROVIDER (provider));
+    g_return_if_fail (shortcut);
+    g_return_if_fail (screen_info);
+
+    sc = xfce_shortcuts_provider_get_shortcut (provider, shortcut);
+
+    if (sc == NULL)
+    {
+        return;
+    }
+
+    dpy = myScreenGetXDisplay (screen_info);
+
+    for (i = 0; i < KEY_COUNT; ++i)
+    {
+        if (g_str_equal (screen_info->params->keys[i].internal_name, sc->command))
+        {
+            parseKeyString (dpy, &screen_info->params->keys[i], shortcut);
+
+            myScreenUngrabKeys (screen_info);
+            myScreenGrabKeys (screen_info);
+            break;
+        }
+    }
+
+    xfce_shortcut_free (sc);
+}
+
+static void
+cb_shortcut_removed (XfceShortcutsProvider *provider, const gchar *shortcut,
+                     ScreenInfo *screen_info)
+{
+    MyKey key;
+    Display *dpy;
+    int i;
+
+    g_return_if_fail (XFCE_IS_SHORTCUTS_PROVIDER (provider));
+    g_return_if_fail (screen_info);
+    g_return_if_fail (shortcut);
+
+    dpy = myScreenGetXDisplay (screen_info);
+
+    parseKeyString (dpy, &key, shortcut);
+
+    for (i = 0; i < KEY_COUNT; ++i)
+    {
+        if (screen_info->params->keys[i].keycode == key.keycode &&
+            screen_info->params->keys[i].modifier == key.modifier)
+        {
+            screen_info->params->keys[i].keycode = 0;
+            screen_info->params->keys[i].modifier = 0;
+
+            myScreenUngrabKeys (screen_info);
+            myScreenGrabKeys (screen_info);
+            break;
+        }
+    }
+}
+
+static void
+parseShortcut (ScreenInfo *screen_info, int index, const gchar *name,
+               GList *shortcuts)
+{
+    Display *dpy;
+    const gchar *shortcut;
+
+    g_return_if_fail (screen_info);
+    g_return_if_fail (index >= 0 && index < KEY_COUNT);
+
+    dpy = myScreenGetXDisplay (screen_info);
+    shortcut = getShortcut (name, shortcuts);
+    parseKeyString (dpy, &screen_info->params->keys[index], shortcut);
+
+    screen_info->params->keys[index].internal_name = g_strdup (name);
+}
+
+static const gchar *
+getShortcut (const gchar *name, GList *shortcuts)
+{
+    XfceShortcut *shortcut;
+    GList *iter;
+    const gchar *result = NULL;
+
+    for (iter = shortcuts; iter != NULL; iter = g_list_next (iter))
+    {
+        shortcut = iter->data;
+
+        if (g_str_equal (shortcut->command, name))
+        {
+            result = shortcut->shortcut;
+            break;
+        }
+    }
+
+    return result;
+}
