@@ -1,4 +1,4 @@
-/*      $Id: workspaces.h 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: workspaces.h 28365 2008-10-22 20:10:00Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
@@ -23,7 +25,7 @@
 #define INC_WORKSPACES_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -35,21 +37,27 @@
 #include "screen.h"
 #include "client.h"
 
-gboolean                workspaceMove                           (ScreenInfo *, 
-                                                                 int, 
-                                                                 int, 
+gboolean                workspaceMove                           (ScreenInfo *,
+                                                                 int,
+                                                                 int,
                                                                  Client *,
                                                                  guint32);
-void                    workspaceSwitch                         (ScreenInfo *, 
-                                                                 int, 
-                                                                 Client *, 
+void                    workspaceSwitch                         (ScreenInfo *,
+                                                                 int,
+                                                                 Client *,
                                                                  gboolean,
                                                                  guint32);
-void                    workspaceSetNames                       (ScreenInfo *, 
-                                                                 gchar **, 
+void                    workspaceSetNames                       (ScreenInfo *,
+                                                                 gchar **,
                                                                  int);
-void                    workspaceSetCount                       (ScreenInfo *, 
+void                    workspaceSetCount                       (ScreenInfo *,
                                                                  int);
 void                    workspaceUpdateArea                     (ScreenInfo *);
 
+void                    workspaceInsert                         (ScreenInfo *,
+                                                                 int);
+
+void                    workspaceDelete                         (ScreenInfo *,
+                                                                 int);
+
 #endif /* INC_WORKSPACES_H */
